/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _ONDEVICESPEECHENGINE_H_
#define _ONDEVICESPEECHENGINE_H_

#include <kylin-ai/ai-engine/large-model/endsidespeechengine.h>
#include <kylin-ai/ai-engine/large-model/error.h>

namespace ai_engine::lm::speech {
class OndeviceSpeechEngine final : public AbstractEndSideSpeechEngine {
public:
    OndeviceSpeechEngine();
    ~OndeviceSpeechEngine() override;

    std::string backendName() override;

    void setRecognizingCallback(
        ai_engine::lm::speech::RecognitionResultCallback callback) override;
    void setRecognizedCallback(
        ai_engine::lm::speech::RecognitionResultCallback callback) override;

    void setSynthesizingCallback(
        ai_engine::lm::speech::SynthesisResultCallback callback) override;
    void setSynthesizedCallback(
        ai_engine::lm::speech::FinishedCallback callback) override;

    // 流式语音识别
    ai_engine::ErrorCode initContinuousRecognitionModule() override;
    ai_engine::ErrorCode
    startContinuousRecognition(const std::string &params) override;
    ai_engine::ErrorCode writeContinuousRecognitionAudioData(
        const std::vector<char> &audioData) override;
    ai_engine::ErrorCode stopContinuousRecognition() override;
    ai_engine::ErrorCode destroyContinuousRecognitionModule() override;

    // 非流式语音识别
    ai_engine::ErrorCode initRecognizeOnceModule() override;
    ai_engine::ErrorCode
    recognizeOnce(const std::string &params,
                  const std::vector<char> &audioData) override;
    ai_engine::ErrorCode destroyRecognizeOnceModule() override;

    // 流式语音合成
    ai_engine::ErrorCode initContinuousSynthesisModule() override;
    ai_engine::ErrorCode
    startContinuousSynthesis(const std::string &params) override;
    ai_engine::ErrorCode
    writeContinuousSynthesisText(const std::string &text) override;
    ai_engine::ErrorCode stopContinuousSynthesis() override;
    ai_engine::ErrorCode destroyContinuousSynthesisModule() override;

    // 非流式语音合成
    ai_engine::ErrorCode initSynthesizeOnceModule() override;
    ai_engine::ErrorCode synthesizeOnce(const std::string &params,
                                        const std::string &text) override;
    ai_engine::ErrorCode destroySynthesizeOnceModule() override;

    // 关键字识别
    ai_engine::ErrorCode initContinuousKeywordRecognitionModule() override;
    ai_engine::ErrorCode
    startContinuousKeywordRecognition(const std::string &params) override;
    ai_engine::ErrorCode writeContinuousKeywordRecognitionAudioData(
        const std::vector<char> &audioData) override;
    ai_engine::ErrorCode stopContinuousKeywordRecognition() override;
    ai_engine::ErrorCode destroyContinuousKeywordRecognitionModule() override;

private:
    uint sessionId_ = 0;
    uint taskid_ = 0;
};
} // namespace ai_engine::lm::speech

#endif